    /************************************************************************\
     *      shortest.c                                                      *
     *      calcolo dello shortest path con l'algoritmo di Dijkstra         *
     *      Enrico Vicario - Ottobre 99                                     *
    \************************************************************************/

#include<stdio.h>

typedef int Boolean;
#define FALSE 1
#define TRUE 0

#define INFINITY 1000000

struct inner_record
/* record di rappresentazione di un elemento nell'iniusme dei vertici raggiunti*/
{
  int index; /*indice del vertice nella matrice di incidenza*/
  int predecessor; /*indice del predecessore lungo il cammino minimo dalla base*/
  float length; /*lunghezza del cammino minimo dalla base*/
};

struct outer_record
/* record di rappresentazione di un elemento nell'insieme dei vertici da raggiungere*/
{
  int nearest_inner; /*indice del vertice piu' prossimo nell'insieme raggiunto*/
  float distance; /*distanza dal vertice piu' prossimo*/
  Boolean valid; /*inizialmente true, vale false quando il vertice
                   rappresentato dal record e' rimosso dall'insieme outer*/
};


void get_arc_values(float * V[], int * N_vertices_ptr);
/* legge da file la dimensione del problema
   alloca e legge la matrice di adiacenza*/

void compute_shortest_paths(float *V, int N_vertices, struct inner_record ** inner_ptr);
/*calcola gli shortest paths con l'algoritmo di Dijkstra e li restituisce
  sull'array puntato da inner_ptr
  Provvede all'allocazione dell'array puntato da inner_ptr*/

void put_shortest_paths(int N_vertices, struct inner_record * inner);
/*scarica su file la matrice dei risultati (inner) */


void main(void)
{

   float * V;  /*matrice di incidenza (valori degli archi)*/

   int N_vertices;
   struct inner_record * path;

   get_arc_values(&V, &N_vertices);
   compute_shortest_paths(V, N_vertices, &path);
   put_shortest_paths(N_vertices,path);
}

void compute_shortest_paths(float *V, int N_vertices, struct inner_record ** path_ptr)
/*calcola gli shortest paths con l'algoritmo di Dijkstra e li restituisce
  sull'array puntato da inner_ptr
  Provvede all'allocazione dell'array puntato da inner_ptr*/
{
   int k,h;

   float best_distance;
   int best_outer;

   struct inner_record * inner; /*array dei descrittori dei vertici raggiunti in ordine di distanza*/
   struct outer_record * outer; /*array dei descrittori dei vertici da raggiungere*/

   /*inizializzazione: inner contiene solo il vertice di base che ha indice 0
                       outer contiene tutti gli altri vertici
                       inner e' un temporaneo che sostituisce (*path_ptr)*/
   inner=(struct inner_record *)malloc(sizeof(struct inner_record)*N_vertices);
   outer=(struct outer_record *)malloc(sizeof(struct outer_record)*N_vertices);

   for(k=0;k<N_vertices;k++)
   {  outer[k].nearest_inner=0;
      outer[k].distance=V[k];  /* V[0][k]==V[0*N_vertices+k] */
      outer[k].valid=TRUE;
   }
   inner[0].index=0;
   inner[0].predecessor=0;
   inner[0].length=0;
   outer[0].valid=FALSE;


   /*main loop: al termine del passo k ho calcolato in inner i primi k vertici piu' vicini alla base */
   for(k=1;k<N_vertices;k++)
   {
         /*cerco il vertice non raggiunto piu' vicino*/
         best_distance=INFINITY;
         for(h=1;h<N_vertices;h++)
         {
            if(outer[h].valid==TRUE && outer[h].distance<best_distance)
            {  best_outer=h;
               best_distance=outer[h].distance;
            }
         }

         /*muovo il vertice piu' vicino nell'insieme dei vertici  raggiunti*/
         inner[k].index=best_outer;
         inner[k].predecessor=outer[best_outer].nearest_inner;
         inner[k].length=outer[best_outer].distance;
         outer[best_outer].valid=FALSE;

         /*aggiorno la distanza dall'insieme raggiunto per i vertici non raggiunti */
         for(h=1;h<N_vertices;h++)
         {
            if(outer[h].valid==TRUE &&
               outer[h].distance>V[inner[k].index*N_vertices+h]+inner[k].length) /*V[x][h]*/
            {
               outer[h].nearest_inner=inner[k].index;
               outer[h].distance=V[inner[k].index*N_vertices+h]+inner[k].length;
            }
         }
   }
   (*path_ptr)=inner;
   /*sporco: si potrebeb sostituire inner con (*inner_ptr) in tutta la
     routine e evitare di allocare la variabile temporanea inner */

}


void get_arc_values(float ** V_ptr, int * N_vertices_ptr)
/* legge da file la dimensione del problema
   alloca e legge la matrice di adiacenza*/
{
   FILE * input_handle;
   int i,j;

   input_handle=fopen("input.txt","r");
   fscanf(input_handle,"%d",N_vertices_ptr);
   (*V_ptr)=(float *)malloc(sizeof(float)*(*N_vertices_ptr)*(*N_vertices_ptr));
   for(i=0;i<(*N_vertices_ptr);i++)
   {
      for(j=0;j<(*N_vertices_ptr);j++)
      {
         fscanf(input_handle,"%f",&(*V_ptr)[i*(*N_vertices_ptr)+j]);   /*V[i][j]*/
         if((*V_ptr)[i*(*N_vertices_ptr)+j]<0||(*V_ptr)[i*(*N_vertices_ptr)+j]>INFINITY)
            (*V_ptr)[i*(*N_vertices_ptr)+j]=INFINITY;
      }
   }
   fclose(input_handle);
}

void put_shortest_paths(int N_vertices, struct inner_record * inner)
/*scarica su file la matrice dei risultati (inner) */
{  FILE * output_handle;
   int k;

   output_handle=fopen("output.txt","w");
   fprintf(output_handle,"\n length   : path");
   for(k=0;k<N_vertices;k++)
   {
      fprintf(output_handle,"\n %f :", inner[k].length);
      fprintf(output_handle," %d<-%d<-...", inner[k].index,inner[k].predecessor);
   }
   fclose(output_handle);
}
